/*
 * ratemask - Calculates ratemask used in ip-sysctl icmp_ratemask
 * Copyright (C) 2002  Oskar Andreasson <blueflux@koffein.net>
 *
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,  
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <netinet/ip_icmp.h>
#include "ratemask.h"



int get_echoreply(int verbosity)
{
	int i;

	i = 1<<ICMP_ECHOREPLY;
	if(verbosity>=1)
		printf("echo reply: %i\n", i);
	return(i);
}

int get_dest_unreach(int verbosity)
{
	int i;

	i = 1<<ICMP_DEST_UNREACH;
	if(verbosity>=1)
		printf("dest unreach: %i\n", i);
	return(i);
}

int get_source_quench(int verbosity)
{
	int i;

	i = 1<<ICMP_SOURCE_QUENCH;
	if(verbosity>=1)
		printf("source quench: %i\n", i);
	return(i);
}

int get_redirect(int verbosity)
{
	int i;

	i = 1<<ICMP_REDIRECT;
	if(verbosity>=1)
		printf("redirect: %i\n", i);
	return(i);
}

int get_echo(int verbosity)
{
	int i;

	i = 1<<ICMP_ECHO;
	if(verbosity>=1)
		printf("echo request: %i\n", i);
	return(i);
}

int get_time_exceeded(int verbosity)
{
	int i;

	i = 1<<ICMP_TIME_EXCEEDED;
	if(verbosity>=1)
		printf("time exceeded: %i\n", i);
	return(i);
}

int get_parameterprob(int verbosity)
{
	int i;

	i = 1<<ICMP_PARAMETERPROB;
	if(verbosity>=1)
		printf("parameter problem: %i\n", i);
	return(i);
}

int get_timestamp(int verbosity)
{
	int i;

	i = 1<<ICMP_TIMESTAMP;
	if(verbosity>=1)
		printf("timpestamp request: %i\n", i);
	return(i);
}

int get_timestampreply(int verbosity)
{
	int i;

	i = 1<<ICMP_TIMESTAMPREPLY;
	if(verbosity>=1)
		printf("timpestamp reply: %i\n", i);
	return(i);
}

int get_info_request(int verbosity)
{
	int i;

	i = 1<<ICMP_INFO_REQUEST;
	if(verbosity>=1)
		printf("info request: %i\n", i);
	return(i);
}

int get_info_reply(int verbosity)
{
	int i;

	i = 1<<ICMP_INFO_REPLY;
	if(verbosity>=1)
		printf("info reply: %i\n", i);
	return(i);
}

int get_address(int verbosity)
{
	int i;

	i = 1<<ICMP_ADDRESS;
	if(verbosity>=1)
		printf("address request: %i\n", i);
	return(i);
}

int output_mask_values(int verbosity) 
{
	verbosity+=1;

	(void)get_echoreply(verbosity);
	(void)get_dest_unreach(verbosity);
	(void)get_source_quench(verbosity);
	(void)get_redirect(verbosity);
	(void)get_echo(verbosity);
	(void)get_time_exceeded(verbosity);
	(void)get_parameterprob(verbosity);
	(void)get_timestamp(verbosity);
	(void)get_timestampreply(verbosity);
	(void)get_info_request(verbosity);
	(void)get_info_reply(verbosity);
	(void)get_address(verbosity);

	return (0);
}

int read_ratemask(int verbosity, int ratemask)
{

	char *ratestring, *tmp, *tmp2;
	
	ratestring = malloc(2000);
	tmp2 = malloc(2000);
	tmp = malloc(200);

	if((ratemask & get_echoreply(verbosity)))
	{
		sprintf(tmp, "echo-reply\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_echoreply(verbosity);
	}
	if((ratemask & get_dest_unreach(verbosity)))
	{
		sprintf(tmp, "dest-unreach\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_dest_unreach(verbosity);
	}
	if((ratemask & get_source_quench(verbosity)))
	{
		sprintf(tmp, "source-quench\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_source_quench(verbosity);
	}
	if((ratemask & get_redirect(verbosity)))
	{
		sprintf(tmp, "redirect\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_redirect(verbosity);
	}
	if((ratemask & get_echo(verbosity)))
	{
		sprintf(tmp, "echo-request\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_echo(verbosity);
	}
	if((ratemask & get_time_exceeded(verbosity)))
	{
		sprintf(tmp, "time-exceeded\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_time_exceeded(verbosity);
	}
	if((ratemask & get_parameterprob(verbosity)))
	{
		sprintf(tmp, "parameter-problem\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_parameterprob(verbosity);
	}
	if((ratemask & get_timestamp(verbosity)))
	{
		sprintf(tmp, "timestamp-request\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_timestamp(verbosity);
	}
	if((ratemask & get_timestampreply(verbosity)))
	{
		sprintf(tmp, "timestamp-reply\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_timestampreply(verbosity);
	}
	if((ratemask & get_info_request(verbosity)))
	{
		sprintf(tmp, "info-request\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_info_request(verbosity);
	}
	if((ratemask & get_info_reply(verbosity)))
	{
		sprintf(tmp, "info-reply\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_info_reply(verbosity);
	}
	if((ratemask & get_address(verbosity)))
	{
		sprintf(tmp, "address-request\n");
		sprintf(tmp2, "%s%s", ratestring, tmp);
		sprintf(ratestring, "%s", tmp2);
		ratemask-=get_address(verbosity);
	}

	printf("The ratemask contained the following legal ICMP types:\n");
	printf("%s\n", ratestring);

	if((ratemask>=0))
	{
		printf("ratemask contains illegal ICMP types:\n%i\n", ratemask);
	}

}

int make_ratemask(int verbosity, char *maskopts)
{

	int ratemask=0;
	char *token;

	token = malloc(200);

	while((token = (char *)strsep(&maskopts, ",")) != 0)
	{
		if(strcmp(token, "echo-reply") == 0)
			ratemask+=get_echoreply(verbosity);
		if(strcmp(token, "dest-unreach") == 0)
			ratemask+=get_dest_unreach(verbosity);
		if(strcmp(token, "source-quench") == 0)
			ratemask+=get_source_quench(verbosity);
		if(strcmp(token, "redirect") == 0)
			ratemask+=get_redirect(verbosity);
		if(strcmp(token, "echo-request") == 0)
			ratemask+=get_echo(verbosity);
		if(strcmp(token, "time-exceeded") == 0)
			ratemask+=get_time_exceeded(verbosity);
		if(strcmp(token, "parameter-problem") == 0)
			ratemask+=get_parameterprob(verbosity);
		if(strcmp(token, "timestamp-request") == 0)
			ratemask+=get_timestamp(verbosity);
		if(strcmp(token, "timestamp-reply") == 0)
			ratemask+=get_timestampreply(verbosity);
		if(strcmp(token, "info-request") == 0)
			ratemask+=get_info_request(verbosity);
		if(strcmp(token, "info-reply") == 0)
			ratemask+=get_info_reply(verbosity);
		if(strcmp(token, "address-request") == 0)
			ratemask+=get_address(verbosity);
	}

	if (verbosity>=1)
		printf("---------------\n");	
	printf("Final ratemask: %i\n", ratemask);

}


int print_help(int verbosity)
{
	printf("ratemask arguments:\n");
	printf("-h	print this help\n");
	printf("-s	show all possible ICMP ratemask values\n");
	printf("-m	make ratemask, followed by these options, separated by comma, no space:\n");
	printf("		echo-reply\n");
	printf("		dest-unreach\n");
	printf("		source-quench\n");
	printf("		redirect\n");
	printf("		echo-request\n");
	printf("		time-exceeded\n");
	printf("		parameter-problem\n");
	printf("		timestamp-request\n");
	printf("		timestamp-reply\n");
	printf("		info-request\n");
	printf("		info-reply\n");
	printf("		address-request\n");
}

int print_version(int verbosity)
{
	printf("ratemask version %s\n", RATEMASK_VERSION);


}

int main (argc,argv,envp) 
int argc;
char *argv[], *envp[];
{

	int i, verbose=0;

	for (i=0; i<=argc-1; i++)
	{
		if (strstr(argv[i], "-v"))
		{
			verbose+=1;
		}
	}
	
	for (i=0; i<=argc-1; i++)
	{

		if (strstr(argv[i], "-h"))
		{
			print_version(verbose);
			print_help(verbose);
			exit(0);
		}

		if (strstr(argv[i], "-V"))
		{
			print_version(verbose);
			exit(0);
		}

		if (strstr(argv[i], "-s"))
		{
			output_mask_values(verbose);
			exit(0);
		}

		if (strstr(argv[i], "-m"))
		{
			make_ratemask(verbose, argv[i+1]);
		}

		if (strstr(argv[i], "-r"))
		{
			read_ratemask(verbose, atoi(argv[i+1]));
		}
	}

	if (argc==1)
	{
		print_help(verbose);
	}

return (0);

}
